local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local Option = BaikCore.Option
local Database = BaikCore.Database

-- Add Module to BaikCore
local Debug = BaikCore:NewModule("Debug")
BaikCore.Debug = Debug

-- Constants
Debug.OFF = 0
Debug.WARN = 1
Debug.DEBUG = 2
Debug.INFO = 3
Debug.VERBOSE = 4
Debug.FIRST = Debug.OFF
Debug.LAST = Debug.VERBOSE

local DEBUG_NAMES = {
    [Debug.OFF] = "Off",
    [Debug.WARN] = "Warn",
    [Debug.DEBUG] = "Debug",
    [Debug.INFO] = "Info",
    [Debug.VERBOSE] = "Verbose"
}

local DEBUG_DB_NAME = "debug"
local DEBUG_DEFAULT = Debug.OFF

local function GetOpts()
    return {
        name = "Debug Level",
        type = "select",
        style = "dropdown",
        values = DEBUG_NAMES,
        get = function(...)
            return Debug:GetLevel()
        end,
        set = function(info, value)
            Database:Set(DEBUG_DB_NAME, value)
        end
    }
end

-- Private Methods
local function ValidateDebugLevel(debugLevel)
    Assert:Number(debugLevel)
    Assert:True(debugLevel >= Debug.OFF and debugLevel <= Debug.VERBOSE,
                "Debug level is not valid")
end

local function IsLevel(value, level)
    return value >= level
end

-- Public API
function Debug:ToString(debugLevel)
    -- Default to member variable
    if not debugLevel then
        debugLevel = Database:Get(DEBUG_DB_NAME)
    end
    -- Enusre the value is valid
    ValidateDebugLevel(debugLevel)

    return DEBUG_NAMES[debugLevel]
end

function Debug:GetLevel()
    return Database:Get(DEBUG_DB_NAME)
end

function Debug:IsWarn()
    return IsLevel(self:GetLevel(), Debug.WARN)
end

function Debug:IsDebug()
    return IsLevel(self:GetLevel(), Debug.DEBUG)
end

function Debug:IsInfo()
    return IsLevel(self:GetLevel(), Debug.INFO)
end

function Debug:IsVerbose()
    return IsLevel(self:GetLevel(), Debug.VERBOSE)
end

-- Ace3 Callbacks
function Debug:OnInitialize()
    Database:InitIfEmpty(DEBUG_DB_NAME, DEBUG_DEFAULT)
    Option:AddOption("general", "debug", GetOpts())
end

function Debug:OnEnable()
end

function Debug:OnDisable()
end
